/*
    kDriver - PC keyboard driver for the TI-89
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _NO_INCLUDE_PATCH

#include "config.h"

#include <link.h>
#include <peekpoke.h>
#include <system.h>

#define LINK_CONTROL 0x60000C
#define LINK_DATA 0x60000E

#define LINK_TO_DATA 0
#define LINK_TO_CLOCK 1
#define LINK_FROM_DATA 2
#define LINK_FROM_CLOCK 3

#define CLOCK_TIMEOUT 810

// peek_bit true => 0V false => 3.3V

void link_init()
{
  // enable low-level access
  pokeIO(LINK_CONTROL, 0);
  pokeIO_bset(LINK_CONTROL, 6);
  pokeIO(LINK_DATA, 0);
  pokeIO_bset(LINK_DATA, 6);

  // set the data line floating and the clock line low (not ready)
  pokeIO_bclr(LINK_DATA, LINK_TO_DATA);
  pokeIO_bset(LINK_DATA, LINK_TO_CLOCK);
}

unsigned char highlow()
{
  unsigned short i;
  // wait for the rising edge of the clock line
  i = 0;
  while(peekIO_bit(LINK_DATA, LINK_FROM_CLOCK)) { if((i++)>CLOCK_TIMEOUT) return TRUE; }
  i = 0;
  // wait for the falling edge of the clock line
  while(!peekIO_bit(LINK_DATA, LINK_FROM_CLOCK)) { if((i++)>CLOCK_TIMEOUT) return TRUE; }
  return FALSE;
}

short receive_byte()
{
  unsigned b;
  unsigned short i;

  b = 0;

  // Disable all interrupts so that they don't mess up our time-critical code
  OSSetSR(0x0700);

  // set the clock line floating, telling the keyboard that we are ready
  pokeIO_bclr(LINK_DATA, LINK_TO_CLOCK);

  // wait for the falling edge of the clock line
  i = 0;
  while(!peekIO_bit(LINK_DATA, LINK_FROM_CLOCK)) { if((i++)>CLOCK_TIMEOUT) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return -1; }}

  for(i=0; i<8; i++) {
    if(highlow()) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return -1; }
    // set the bit according to the data line
    if(!peekIO_bit(LINK_DATA, LINK_FROM_DATA)) b |= (1 << i);
  }

  // parity and stop bits are ignored for now
  if(highlow()) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return -1; }
  if(highlow()) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return -1; }

  // set status to not ready
  pokeIO_bset(LINK_DATA, LINK_TO_CLOCK);

  OSSetSR(0x0000);
  return b;
}

unsigned char send_byte(unsigned char b)
{
  unsigned char parity;
  unsigned short i;

  parity = 0;

  // Disable all interrupts so that they don't mess up our time-critical code
  OSSetSR(0x0700);

  // Set the data line low and the clock floating ; the keyboard should start generating the clock
  pokeIO_bset(LINK_DATA, LINK_TO_DATA);
  pokeIO_bclr(LINK_DATA, LINK_TO_CLOCK);

  // wait for the falling edge of the clock line
  i = 0;
  while(!peekIO_bit(LINK_DATA, LINK_FROM_CLOCK)) { if((i++)>CLOCK_TIMEOUT) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return TRUE; }}

  for(i=0;i<8;i++) {
    // set the data bit
    if((b & (1 << i)) != 0) {
      pokeIO_bclr(LINK_DATA, LINK_TO_DATA);
      parity++;
    } else {
      pokeIO_bset(LINK_DATA, LINK_TO_DATA);
    }
    if(highlow()) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return TRUE; }
  }
  // send the parity bit
  if((parity & 1) == 1) pokeIO_bset(LINK_DATA, LINK_TO_DATA); else pokeIO_bclr(LINK_DATA, LINK_TO_DATA);
  if(highlow()) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return TRUE; }
  // release the data line
  pokeIO_bclr(LINK_DATA, LINK_TO_DATA);
  if(highlow()) { pokeIO_bset(LINK_DATA, LINK_TO_CLOCK); OSSetSR(0x0000); return TRUE; }
    // ack is ignored

  // set status to not ready
  pokeIO_bset(LINK_DATA, LINK_TO_CLOCK);

  OSSetSR(0x0000);
  return 0;
}

void link_done()
{
  OSLinkReset();
}

